# -*- coding: utf-8 -*-

import os
import GMain
import gtk
import gobject
import googsystray
import sys

try:
	import win32crypt
except:
	pass

try:
	import gnomekeyring as gkey
	_has_gkeyring = True
except:
	_has_gkeyring = False
	
try:
	import dbus
	_has_dbus = True
	_bus = dbus.SessionBus()
except:
	_has_dbus = False
	
class GKeyring:
	def get_creds(self, service, config):
		return (None, None)
		
	def set_creds(self, service, config, username, password):
		return
		
	def __init__ (self):
		return
		

class GKeyringGnome (GKeyring):
	def get_creds(self, service, config):
		attrs = {}
		attrs["server"] = service.encode("utf-8")

		try:
			items = gkey.find_items_sync(gkey.ITEM_NETWORK_PASSWORD, attrs)
			return (items[0].attributes["user"], items[0].secret)
		except gkey.NoMatchError, e:
			googsystray.error("NO MATCH")
		except gkey.DeniedError, e:
			googsystray.error("ERROR FETCHING PASSWORDS")
		except gkey.CancelledError, e:
			googsystray.error("CANCELLEDERROR")
		return (None, None)
		
	def set_creds(self, service, config, username, password):
		attrs = { "server": service,
		          "user": username,}
		gkey.item_create_sync(self._keyring, gkey.ITEM_NETWORK_PASSWORD, "googsystray", attrs, password, True)
		return
		
	def __init__ (self):
		GKeyring.__init__(self)
		self._keyring = gkey.get_default_keyring_sync()
		
class GKeyringKDE (GKeyring):
	def get_creds(self, service, config):
		if self.hasFolder(self.handle, self.appid, self.appid) == 0:
			return (None, None)
		
		password = self.readPassword(self.handle, self.appid, service + "_pw", self.appid)
		username = self.readPassword(self.handle, self.appid, service + "_un", self.appid)
				
		return (username, password)
		
	def set_creds(self, service, config, username, password):
		if self.hasFolder(self.handle, self.appid, self.appid) == 0:
			self.createFolder(self.handle, self.appid, self.appid)
		
		
		self.writePassword(self.handle, self.appid, service + "_pw", password, self.appid)
		self.writePassword(self.handle, self.appid, service + "_un", username, self.appid)
		
		return
	
	def __init__ (self):
		GKeyring.__init__(self)
		# 1234?  So cheating.  Fixme, do this right.  Does it matter?
		self.id = 1234
		self.appid = "Googsystray"
		self.proxy = _bus.get_object("org.kde.kwalletd", "/modules/kwalletd")
		self.open = self.proxy.get_dbus_method("open", "org.kde.KWallet")
		self.hasFolder = self.proxy.get_dbus_method("hasFolder", "org.kde.KWallet")
		self.createFolder = self.proxy.get_dbus_method("createFolder", "org.kde.KWallet")
		self.writePassword = self.proxy.get_dbus_method("writePassword", "org.kde.KWallet")
		self.readPassword = self.proxy.get_dbus_method("readPassword", "org.kde.KWallet")
		
		self.handle = self.open("kdewallet", self.id, self.appid)
		# Catch failure case!!!!!!! fixme
		

#from pywin
def str2bytes(sval):
	if sys.version_info < (3,0) and isinstance(sval, str):
        	sval = sval.decode("latin1")
	return sval.encode("latin1")
                

class GKeyringW32 (GKeyring):
	def get_creds(self, service, config):
		if "bpassword" not in config[service]:
			return (None, None)

		bin_pw = eval(config[service]["bpassword"])

		username, password = win32crypt.CryptUnprotectData(bin_pw, None, None, None, 0)

		return (username, password)
		
	def set_creds(self, service, config, username, password):
	
		pw = str2bytes(password)
		blob = win32crypt.CryptProtectData(pw, username, None, None, None, 0)
		bin_pw = repr(blob)

		config[service]["bpassword"] = bin_pw
		
		return
		
	def __init__ (self):
		GKeyring.__init__(self)
		



_keyring = None

def detect_keyring():
	if os.name == "nt":
		return GKeyringW32()
	else:
		if not _has_dbus:
			if _has_gkeyring:
				# Maybe check to see if it's running?
				return GKeyringGnome()
			return None
		
		
		gnomerunning = False
		# Why do we loop twice?  In case both are running, we default
		# to gnome this way.  Otherwise, we'd just pick a random one.
		for i in _bus.list_names():
			if i == "org.gnome.keyring":
				if _has_gkeyring:
					return GKeyringGnome()
				else:
					gnomerunning = True
					
		for i in _bus.list_names():
			if i == "org.kde.kwalletd":
				return GKeyringKDE()

		# Nothing running.  Fall back to
		# just storing in plaintext.  We could try to get
		# dbus to start one up, but screw that.  If you want
		# keyrings, you ought to be running a key ring manager :)
		
		if "keyring_dlg_dns" in GMain._GConf.gconfig["global"]:
			if GMain._GConf.gconfig["global"]["keyring_dlg_dns"]:
				return
		error_dlg = gtk.Dialog(_("Keyring Error"), None, gtk.DIALOG_MODAL, (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
		
		if gnomerunning:
			label = gtk.Label(_("GNOME Keyring service running, but python support for it is not installed.  Authentication will be stored in plaintext."))
		else:
			label = gtk.Label(_("No keyring service is running.  Authentication will be stored in plaintext."))
			
		label.set_line_wrap(True);
		cbutton = gtk.CheckButton(_("Do not show this dialog again."))
		error_dlg.get_content_area().pack_start(label, True, True, 5);
		error_dlg.get_content_area().pack_start(cbutton, False, False, 5);
		error_dlg.show_all()
		error_dlg.run()
		if cbutton.get_active():
			GMain._GConf.gconfig["global"]["keyring_dlg_dns"] = True
			GMain._GConf.save_config()
		error_dlg.destroy()			

	return None


def get_creds(service, backup=False):
	global _keyring
	if backup:
		config = GMain._GConf.backupconfig["global"]["accounts"]
	else:
		config = GMain._GConf.gconfig["global"]["accounts"]
	if _keyring == None:
		_keyring = detect_keyring()
	if _keyring == None:
		username = config[service]["username"]
		password = config[service]["password"]
		return (username, password)
		
	if config[service]["username"] != "KEYRING": 
		username = config[service]["username"]
		password = config[service]["password"]
		set_creds(service, username, password, backup)
		return (username, password)
		# Let's convert to the keyring.
		
	return _keyring.get_creds(service, config)


def set_creds(service, username, password, backup=False):
	global _keyring
	if backup:
		config = GMain._GConf.backupconfig["global"]["accounts"]
	else:
		config = GMain._GConf.gconfig["global"]["accounts"]
	
	if _keyring == None:
		_keyring = detect_keyring()

	if _keyring == None:
		config[service]["username"] = username
		config[service]["password"] = password
		if not backup:
			GMain._GConf.save_config()
		return

	_keyring.set_creds(service, config, username, password)
	config[service]["username"] = "KEYRING"
	config[service]["password"] = "KEYRING"
	if not backup:
		GMain._GConf.save_config()


	return
